@@ -130,6 +130,8 @@ evalAndDumpToDB <- function(db, expr, exprDigest) {
         ## Dump the values of the keys to the database
         dumpToDB(db, list = keys, envir = env)
 
+	if(length(keys) > 0)
+		copy2env(keys, env, globalenv())
         keys
 }
 
@@ -141,6 +143,15 @@ mangleDigest <- function(x) {
         paste(".__", x, "__", sep = "")
 }
 
+hash <- function(object) {
+	digest(object, algo = "sha1")
+}
+
+hashExpr <- function(expr) {
+	expr <- deparse(expr, width.cutoff = 60)
+	hash(expr)
+}
+
 ################################################################################
 ## The major modification is here: Rather than evaluate expressions
 ## and leave them in the global environment, we evaluate them in a
@@ -169,7 +180,7 @@ cacheSweaveEvalWithOpt <- function (expr, options) {
                 ## Create database name from chunk label and MD5
                 ## digest
                 dbName <- makeChunkDatabaseName(cachedir, options, chunkDigest)
-                exprDigest <- mangleDigest(digest(expr))
+                exprDigest <- mangleDigest(hashExpr(expr))
 
                 ## Create 'stashR' database
                 db <- new("localDB", dir = dbName, name = basename(dbName))
@@ -179,19 +190,21 @@ cacheSweaveEvalWithOpt <- function (expr, options) {
                 ## objects to the database.  Otherwise, just read the
                 ## vector of keys from the database
 
-                keys <- if(!dbExists(db, exprDigest)) 
-                        try({
+                if(!dbExists(db, exprDigest)) {
+                        keys <- try({
                                 evalAndDumpToDB(db, expr, exprDigest)
                         }, silent = TRUE)
-                else 
-                        dbFetch(db, exprDigest)
 
-                ## If there was an error then just return the
-                ## condition object and let Sweave deal with it.
-                if(inherits(keys, "try-error"))
-                        return(keys)
-
-                dbLazyLoad(db, globalenv(), keys)
+			## If there was an error then just return the
+			## condition object and let Sweave deal with it.
+			if(inherits(keys, "try-error"))
+				return(keys)
+		}
+		else {
+                        keys <- dbFetch(db, exprDigest)
+			dbLazyLoad(db, globalenv(), keys)
+		}
+		keys
         }
         else {
                 ## If caching is turned off, just evaluate the expression
@@ -223,67 +236,16 @@ cacheSweaveSetup <- function(file, syntax,
         ## Add the (non-standard) options for code chunks with caching
         out$options[["cache"]] <- cache
 
-        ## We assume that each .Rnw file gets its own map file
-        out[["mapFile"]] <- makeMapFileName(file)
-        file.create(out[["mapFile"]])  ## Overwrite an existing file
-
         ## End additions [RDP]
 ######################################################################
         out
 }
 
 
-makeMapFileName <- function(Rnwfile) {
-        mapfile <- sub("\\.Rnw$", "\\.map", Rnwfile)
-
-        ## Don't clobber
-        if(identical(mapfile, Rnwfile))
-                mapfile <- paste(Rnwfile, "map", sep = ".")
-        mapfile
-}
-
-writeChunkMetadata <- function(object, chunk, options) {
-        chunkprefix <- utils::RweaveChunkPrefix(options)
-        chunkexps <- parse(text = chunk)
-        chunkDigest <- digest(chunkexps)
-
-        options$chunkDigest <- chunkDigest
-        
-        ## If there's a data map file then write the chunk name and the
-        ## directory of the chunk database to the map file (in DCF format)
-        dbName <- if(isTRUE(options$cache))
-                makeChunkDatabaseName(getCacheDir(), options, chunkDigest)
-        else
-                ""
-        ## Capture figure filenames; default to PDF, otherwise use EPS.
-        ## Filenames are <chunkprefix>.<extenstion>, which could change in
-        ## the future depending on Sweave implementation details
-        figname <- ""
-        if(options$fig && options$eval) {
-                figname <- if(options$pdf)
-                        paste(chunkprefix, "pdf", sep = ".")
-                else if(options$eps)
-                        paste(chunkprefix, "eps", sep = ".")
-                else
-                        ""
-        }
-        ## Write out map file entry
-        mapFile <- object[["mapFile"]]
-        mapEntry <- data.frame(chunk = options$label,
-                               chunkprefix = chunkprefix,
-                               fig = figname,
-                               cacheDB = dbName,
-                               time = Sys.time())
-        write.dcf(mapEntry, file = mapFile, append = TRUE, width = 2000)
-	cat("\n", file = mapFile, append = TRUE)
-        options
-}
-
 ## This function is essentially unchanged from the original Sweave
-## version, except I compute the digest of the entire chunk, write out
-## information to the map file, and use 'cacheSweaveEvalWithOpt'
-## instead.  Note that everything in this function operates at the
-## chunk level.  The code has been copied from R 2.5.0.
+## version, except I use 'cacheSweaveEvalWithOpt' instead.  Note that
+## everything in this function operates at the chunk level.  The code
+## has been copied from R 2.5.0.
 
 cacheSweaveRuncode <- function(object, chunk, options) {
         if(!(options$engine %in% c("R", "S"))){
@@ -331,10 +293,8 @@ cacheSweaveRuncode <- function(object, chunk, options) {
         chunkexps <- try(parse(text=chunk), silent=TRUE)
         RweaveTryStop(chunkexps, options)
 
-        ## Adding my own stuff here [RDP]
-        ## Add 'chunkDigest' to 'options'
-        options <- writeChunkMetadata(object, chunk, options)
-        ## End adding my own stuff [RDP]
+        ## Additions here [RDP]
+        options$chunkDigest <- hashExpr(parse(text = chunk, srcfile = NULL))
 
         openSinput <- FALSE
         openSchunk <- FALSE
